from __future__ import division

from PIL import Image
import glob, random, os, herrams

FADE = 15
FPS = 60

class Acercamiento(object):
    def __init__(self, im, size, imgid):
        self.paso = ESTABLE + FADE
        self.ind = 0
        self.im = im
        self.id = imgid
        self.size = size

        # see if we should resize this image, and do it
        sizex_should, sizey_should = size
        sizex_this, sizey_this = self.im.size
#        print "tamanio", size, self.im.size
        if abs(sizey_this - sizey_should) / sizey_should > 0.01:
            # calculate new size going for full height, but keeping
            # the form factor
            new_x = int(round(sizey_should * sizex_this / sizey_this))
            self.im = self.im.resize((new_x, sizey_should), Image.BICUBIC)
#            print "tamanio2", size, self.im.size

        # if it's vertical, let's center it
        sizex_this, sizey_this = self.im.size
        if sizey_this > sizex_this:
            y0 = 0  # the bottom of the image
            y1 = sizey_this  # the top of the image
            vertical_band = (sizex_should - sizex_this) / 2
            x0 = -vertical_band
            x1 = sizex_this + vertical_band
            self.im = self.im.transform(size, Image.EXTENT, (x0, y0, x1, y1))

        fsi = random.random() * .2
#        print "========= imgid", imgid, fsi
        fid = .2 - fsi
        self.x0 = size[0] * fsi
        self.y0 = size[1] * fsi
        self.x1 = size[0] * fid
        self.y1 = size[1] * fid
#        print "random", self.x0, self.y0, self.x1, self.y1

    def mix(self):
        rx0 = self.x0 - (self.x0*self.ind/self.paso)
        ry0 = self.y0 - (self.y0*self.ind/self.paso)
        rx1 = self.size[0] - (self.x1 - (self.x1*self.ind/self.paso))
        ry1 = self.size[1] - (self.y1 - (self.y1*self.ind/self.paso))
#        print "acerc", self.id, rx0, ry0, rx1, ry1
        n = self.im.transform(self.size, Image.EXTENT, (rx0, ry0, rx1, ry1))
        self.ind += 1
#        print "ind", self.ind
        return n


def blend(img1, img2, img3):
    factor = .5
    while factor < 1:
        mezc = Image.blend(img1.mix(), img2.mix(), factor)
        img = mezc
        yield img
        factor += 1/FADE

    for x in range(ESTABLE):
        yield img2.mix()

    factor = 0
    while factor < .5:
        mezc = Image.blend(img2.mix(), img3.mix(), factor)
        img = mezc
        yield img
        factor += 1/FADE


def compone(fondobase, imagenes):
    m1 = fondobase
    esa = imagenes.next()
    for img in imagenes:
        for frame in blend(m1, esa, img):
            yield frame
        m1 = esa
        esa = img

    for frame in blend(m1, esa, fondobase):
        yield frame


def main(direct, name, randomize, firstimg,
         time_total=None, time_perimage=None):
    # info del video
    archs = glob.glob(os.path.join(direct, "*.JPEG"))
    archs += glob.glob(os.path.join(direct, "*.jpeg"))
    archs = glob.glob(os.path.join(direct, "*.JPG"))
    archs += glob.glob(os.path.join(direct, "*.jpg"))
    if randomize:
        random.shuffle(archs)
    else:
        archs.sort()
    if firstimg is not None:
        archs.remove(firstimg)
        archs.insert(0, firstimg)
    print "Procesando %d archivos" % len(archs)
    if time_perimage is not None:
        time_total = time_perimage * len(archs)
    print "Estimated total video time:", time_total
    global ESTABLE

    # try to get a size for them all, choose smaller, complain
    # about wrong orientation
    images_raw =  [(Image.open(x), x) for x in archs]
    max_size_x = None
    for img, fname in images_raw:
        sizx, sizy = img.size
        if sizx < sizy:
            continue
        if max_size_x is None or sizx < max_size_x:
            max_size_x = sizx
    size = (max_size_x, int(round(max_size_x * 3 / 4)))
    print "========= final", size

    imagenes = (Acercamiento(im, size, fname) for im, fname in images_raw)

    negra = Image.new("RGB", size, (0,0,0))
    negra.save("negra.jpg", "JPEG")

    # vemos el tiempo
    cant = len(archs) + 1
    ESTABLE = int(round( (FPS * time_total) / cant - FADE ))
    print "Cantidad de frames:  Estable: %d   Fade: %d" % (ESTABLE, FADE)

    # armamos todas los frames
    av = herrams.Avance((FADE+ESTABLE)*cant)
    print "Arrancando"
    if not os.path.exists("tmp_frames"):
        os.mkdir("tmp_frames")
    background = Acercamiento(negra, size, 'negra')
    for i, img in enumerate(compone(background, imagenes)):
        nom = "tmp_frames/comp%08d.jpg" % i
        img.save(nom, "JPEG")
        av.step()

    print "\nGenerando video"
    cmd = 'mencoder "mf://tmp_frames/*.jpg" -mf fps=%d -o tmp.avi -ovc lavc ' +\
          '-lavcopts vcodec=mjpeg'
    os.system(cmd % FPS)
    if os.path.exists(name + ".dv"):
        os.remove(name + ".dv")
    cmd = "ffmpeg -i tmp.avi -target pal-dv %s.dv"
    os.system(cmd % name)

    # remove temp files
    for nomarch in glob.glob("tmp_frames/comp*.jpg"):
        os.remove(nomarch)
    os.rmdir("tmp_frames")
    os.remove("tmp.avi")


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description='Generate the video')
    parser.add_argument('images_directory', help="the directory with images")
    parser.add_argument("--randomize", action='store_true',
                        help="Randomize the images order")
    parser.add_argument("--first", action='store',
                        help="Indicate which is the first image of the video")

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("--total-time", action='store', type=int,
                       help="The approximate total time for the video")
    group.add_argument("--time-per-image", action='store', type=int,
                       help="The time for each image")
    args = parser.parse_args()

    direct = args.images_directory
    if direct.endswith("/"):
        direct = direct[:-1]
    name = os.path.basename(direct)
    main(direct, name, args.randomize, args.first,
         time_total=args.total_time, time_perimage=args.time_per_image)
